==> node-red-contrib-simple-message-queue-0.1.0.tgz: package/simple-message-queue.js <==
/*
  A simpe messages queue for IBM's Node-Red
  https://github.com/shady2k/node-red-contrib-simple-message-queue
  (c) 2017, shady2k <shady2k@gmail.com>
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

module.exports = function(RED) {
	
	function isNormalInteger(str) {
	    return /^\+?(0|[1-9]\d*)$/.test(str);
	}

	function simpleMessageQueueNode(config) {

		RED.nodes.createNode(this, config);
		this.name = config.name;
		this.firstMessageBypass = config.firstMessageBypass || false;
		this.bypassInterval = config.bypassInterval || 0;
		var isFirstMessage = true;
		var node = this;
		
		// Yes it's true: an incoming message just happened
		this.on("input", function(msg) {
			var now = Date.now;
			var context = node.context();

			// if queue doesn't exist, create it
			context.queue = context.queue || [];
			context.is_disabled = context.is_disabled || false;

			// if the msg is a reset, clear queue
			if (msg.hasOwnProperty("reset")) {
		        context.queue = [];
		        isFirstMessage = true;
			} else if (msg.hasOwnProperty("bypass")) {
				if(msg.bypass) {
					context.is_disabled = false;
				} else {
					context.is_disabled = true;
					context.queue = [];
				}
			} else if (msg.hasOwnProperty("trigger")) {   // if the msg is a trigger one release next message
		        // Filter overdue messages
				context.queue = context.queue.filter(function(x) {
					return ((now() - x._queuetimestamp) < x.ttl || x.ttl == 0);
				});
			    if(context.queue.length > 0) {
			        var m = context.queue.shift();
		            node.send(m);
			    }
			} else {
				if(context.is_disabled || (node.firstMessageBypass && isFirstMessage)) {
					isFirstMessage = false;
					node.send(msg);
				} else {
					// Check if ttl value of new message is positive integer
					var ttl = msg.ttl || 0;
					if(!isNormalInteger(ttl)) ttl = 0;

					msg.ttl = ttl;
					msg._queuetimestamp = now();
					context.queue.push(msg); // Add to queue

					// Filter overdue messages
					context.queue = context.queue.filter( function(x) {
						return ((now() - x._queuetimestamp) < x.ttl || x.ttl == 0);
					});

					if(node.bypassInterval > 0) {
						setTimeout(function() {
							if(context.queue.length == 1) {
								var m = context.queue.shift();
								node.send(m);
								// Update status
								node.status({fill:"green",shape:"ring",text: context.queue.length});
							}
						}, node.bypassInterval);
					}
				}
			}

			// Update status
			node.status({fill:"green",shape:"ring",text: context.queue.length});
		});

		this.on("close", function() {
			// Update status
			node.status({fill:"green",shape:"ring",text: 0});
		});
	}

	RED.nodes.registerType("simple-queue", simpleMessageQueueNode);
};

==> node-red-contrib-simple-message-queue-0.2.0.tgz: package/simple-message-queue.js <==
/*
  A simpe messages queue for IBM's Node-Red
  https://github.com/shady2k/node-red-contrib-simple-message-queue
  (c) 2017, shady2k <shady2k@gmail.com>
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

module.exports = function(RED) {
	
	function isNormalInteger(str) {
	    return /^\+?(0|[1-9]\d*)$/.test(str);
	}

	function simpleMessageQueueNode(config) {

		RED.nodes.createNode(this, config);
		this.name = config.name;
		this.firstMessageBypass = config.firstMessageBypass || false;
		this.bypassInterval = config.bypassInterval || 0;
		var isBusy = false;
		var node = this;
		
		// Yes it's true: an incoming message just happened
		this.on("input", function(msg) {
			var now = Date.now;
			var context = node.context();

			// if queue doesn't exist, create it
			context.queue = context.queue || [];
			context.is_disabled = context.is_disabled || false;

			// if the msg is a reset, clear queue
			if (msg.hasOwnProperty("reset")) {
		        context.queue = [];
		        isBusy = false;
			} else if (msg.hasOwnProperty("bypass")) {
				if(msg.bypass) {
					context.is_disabled = true;
				} else {
					context.is_disabled = false;
					isBusy = false;
					context.queue = [];
				}
			} else if (msg.hasOwnProperty("trigger")) {   // if the msg is a trigger one release next message
		        // Filter overdue messages
				context.queue = context.queue.filter(function(x) {
					return ((now() - x._queuetimestamp) < x.ttl || x.ttl == 0);
				});
			    if(context.queue.length > 0) {
			        var m = context.queue.shift();
		            node.send(m);

					if(node.bypassInterval > 0) {
						setTimeout(function bypassSend() {
							if(context.queue.length > 0) {
								var m = context.queue.shift();
								node.send(m);
								if(context.queue.length == 0) {
									isBusy = false;
								}
								setTimeout(bypassSend, node.bypassInterval);

								// Update status
								node.status({fill:"green",shape:"ring",text: context.queue.length});
							}
						}, node.bypassInterval);
					}
			    } else {
			    	isBusy = false;
			    }
			} else {
				if(context.is_disabled || (node.firstMessageBypass && !isBusy)) {
					isBusy = true;
					node.send(msg);

					if(node.bypassInterval > 0) {
						setTimeout(function bypassSend() {
							if(context.queue.length > 0) {
								var m = context.queue.shift();
								node.send(m);
								if(context.queue.length == 0) {
									isBusy = false;
								}
								setTimeout(bypassSend, node.bypassInterval);

								// Update status
								node.status({fill:"green",shape:"ring",text: context.queue.length});
							}
						}, node.bypassInterval);
					}
				} else {
					// Check if ttl value of new message is positive integer
					var ttl = msg.ttl || 0;
					if(!isNormalInteger(ttl)) ttl = 0;

					msg.ttl = ttl;
					msg._queuetimestamp = now();
					context.queue.push(msg); // Add to queue

					// Filter overdue messages
					context.queue = context.queue.filter( function(x) {
						return ((now() - x._queuetimestamp) < x.ttl || x.ttl == 0);
					});
				}
			}

			// Update status
			node.status({fill:"green",shape:"ring",text: context.queue.length});
		});

		this.on("close", function() {
			// Update status
			node.status({fill:"green",shape:"ring",text: 0});
		});
	}

	RED.nodes.registerType("simple-queue", simpleMessageQueueNode);
};
